#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <ctype.h>

#include "../../config.h"
#include "../../include/types.h"
#include "../../include/debug.h"

#include "../../include/timing.h"

#include "../../include/mem.h"
#include "../../include/gfx.h"
#include "../../include/dev.h"

#include "../../include/core.h"

#include "z80.h"
#include "flags.h"

#define INTERNAL static inline

typedef struct Z80_instruction {
	const char *id;
	unsigned char size;
	unsigned char code[4];
} z80_instruction;

INTERNAL int _strin(const char *big, const char *little)
{
	char *fnd = strstr(big, little);

	if (fnd == NULL) {
		return -1;
	} else {
		return (int)(fnd - big);
	}
}

INTERNAL char *_left(const char *str, int len)
{
	char *ret;

	if (strlen(str) < len)
		len = strlen(str);

	ret = (char *)malloc(sizeof(char) * len+1);
	if (ret == NULL) {
		fprintf(stderr, "OMG!\n");
		exit(42);
	}

	strncpy(ret, str, len);
	ret[len] = 0;

	return ret;
}

INTERNAL char *_right(const char *str, int len)
{
	char *ret;

	if (strlen(str) < len)
		ret = strdup("");
	else
		ret = strdup(str+len);

	if (ret == NULL) {
		fprintf(stderr, "OMG!\n");
		exit(42);
	}

	return ret;
}

static z80_instruction Z80_instructs[] = {
  { "ld bc, !2",      1, { 0x01, 0x00, 0x00, 0x00 } },
  { "ld de, !2",      1, { 0x11, 0x00, 0x00, 0x00 } },
  { "ld hl, !2",      1, { 0x21, 0x00, 0x00, 0x00 } },
  { "ld ix, !2",      2, { 0xDD, 0x21, 0x00, 0x00 } },
  { "ld iy, !2",      2, { 0xFD, 0x21, 0x00, 0x00 } },
  { "ld sp, !2",      1, { 0x31, 0x00, 0x00, 0x00 } },

  { "ld (bc), a",     1, { 0x02, 0x00, 0x00, 0x00 } },
  { "ld (de), a",     1, { 0x12, 0x00, 0x00, 0x00 } },
  { "ld a, (bc)",     1, { 0x0A, 0x00, 0x00, 0x00 } },
  { "ld a, (de)",     1, { 0x1A, 0x00, 0x00, 0x00 } },
  { "ld (!2), hl",    1, { 0x22, 0x00, 0x00, 0x00 } },
  { "ld (!2), ix",    2, { 0xDD, 0x22,  0x00, 0x00 } },
  { "ld (!2), iy",    2, { 0xFD, 0x22,  0x00, 0x00 } },
  { "ld (!2), a",     1, { 0x32, 0x00, 0x00, 0x00 } },
  { "ld hl, (!2)",    1, { 0x2A, 0x00, 0x00, 0x00 } },
  { "ld ix, (!2)",    2, { 0xDD, 0x2A, 0x00, 0x00 } },
  { "ld iy, (!2)",    2, { 0xFD, 0x2A, 0x00, 0x00 } },
  { "ld a, (!2)",     1, { 0x3A, 0x00, 0x00, 0x00 } },

  { "ld bc, (!2)",    2, { 0xED, 0x4B, 0x00, 0x00 } },
  { "ld de, (!2)",    2, { 0xED, 0x5B, 0x00, 0x00 } },
  { "ld hl, (!2)",    2, { 0xED, 0x6B, 0x00, 0x00 } },
  { "ld sp, (!2)",    2, { 0xED, 0x7B, 0x00, 0x00 } },

  { "ld (!2), bc",    2, { 0xED, 0x43, 0x00, 0x00 } },
  { "ld (!2), de",    2, { 0xED, 0x53, 0x00, 0x00 } },
  { "ld (!2), hl",    2, { 0xED, 0x63, 0x00, 0x00 } },
  { "ld (!2), sp",    2, { 0xED, 0x73, 0x00, 0x00 } },

  { "inc bc",	 1, { 0x03, 0x00, 0x00, 0x00 } },
  { "inc de",	 1, { 0x13, 0x00, 0x00, 0x00 } },
  { "inc hl",	 1, { 0x23, 0x00, 0x00, 0x00 } },
  { "inc ix",	 2, { 0xDD, 0x23, 0x00, 0x00 } },
  { "inc iy",	 2, { 0xFD, 0x23, 0x00, 0x00 } },
  { "inc sp",	 1, { 0x33, 0x00, 0x00, 0x00 } },

  { "dec bc",	 1, { 0x0B, 0x00, 0x00, 0x00 } },
  { "dec de",	 1, { 0x1B, 0x00, 0x00, 0x00 } },
  { "dec hl",	 1, { 0x2B, 0x00, 0x00, 0x00 } },
  { "dec ix",	 2, { 0xDD, 0x2B, 0x00, 0x00 } },
  { "dec iy",	 2, { 0xFD, 0x2B, 0x00, 0x00 } },
  { "dec sp",	 1, { 0x3B, 0x00, 0x00, 0x00 } },

  { "inc b",	  1, { 0x04, 0x00, 0x00, 0x00 } },
  { "inc c",	  1, { 0x0C, 0x00, 0x00, 0x00 } },
  { "inc d",	  1, { 0x14, 0x00, 0x00, 0x00 } },
  { "inc e",	  1, { 0x1C, 0x00, 0x00, 0x00 } },
  { "inc h",	  1, { 0x24, 0x00, 0x00, 0x00 } },
  { "inc l",	  1, { 0x2C, 0x00, 0x00, 0x00 } },
  { "inc (hl)",       1, { 0x34, 0x00, 0x00, 0x00 } },
  { "inc (ix+!d)",    2, { 0xDD, 0x34, 0x00, 0x00 } },
  { "inc (iy+!d)",    2, { 0xFD, 0x34, 0x00, 0x00 } },
  { "inc a",	  1, { 0x3C, 0x00, 0x00, 0x00 } },

  { "dec b",	  1, { 0x05, 0x00, 0x00, 0x00 } },
  { "dec c",	  1, { 0x0D, 0x00, 0x00, 0x00 } },
  { "dec d",	  1, { 0x15, 0x00, 0x00, 0x00 } },
  { "dec e",	  1, { 0x1D, 0x00, 0x00, 0x00 } },
  { "dec h",	  1, { 0x25, 0x00, 0x00, 0x00 } },
  { "dec l",	  1, { 0x2D, 0x00, 0x00, 0x00 } },
  { "dec (hl)",       1, { 0x35, 0x00, 0x00, 0x00 } },
  { "dec (ix+!d)",    2, { 0xDD, 0x35, 0x00, 0x00 } },
  { "dec (iy+!d)",    2, { 0xFD, 0x35, 0x00, 0x00 } },
  { "dec a",	  1, { 0x3D, 0x00, 0x00, 0x00 } },

  { "ld b, b",	1, { 0x40, 0x00, 0x00, 0x00 } },
  { "ld b, c",	1, { 0x41, 0x00, 0x00, 0x00 } },
  { "ld b, d",	1, { 0x42, 0x00, 0x00, 0x00 } },
  { "ld b, e",	1, { 0x43, 0x00, 0x00, 0x00 } },
  { "ld b, h",	1, { 0x44, 0x00, 0x00, 0x00 } },
  { "ld b, l",	1, { 0x45, 0x00, 0x00, 0x00 } },
  { "ld b, (hl)",     1, { 0x46, 0x00, 0x00, 0x00 } },
  { "ld b, (ix+!d)",  2, { 0xDD, 0x46, 0x00, 0x00 } },
  { "ld b, (iy+!d)",  2, { 0xFD, 0x46, 0x00, 0x00 } },
  { "ld b, a",	1, { 0x47, 0x00, 0x00, 0x00 } },
  { "ld c, b",	1, { 0x48, 0x00, 0x00, 0x00 } },
  { "ld c, c",	1, { 0x49, 0x00, 0x00, 0x00 } },
  { "ld c, d",	1, { 0x4A, 0x00, 0x00, 0x00 } },
  { "ld c, e",	1, { 0x4B, 0x00, 0x00, 0x00 } },
  { "ld c, h",	1, { 0x4C, 0x00, 0x00, 0x00 } },
  { "ld c, l",	1, { 0x4D, 0x00, 0x00, 0x00 } },
  { "ld c, (hl)",     1, { 0x4E, 0x00, 0x00, 0x00 } },
  { "ld c, (ix+!d)",  2, { 0xDD, 0x4E, 0x00, 0x00 } },
  { "ld c, (iy+!d)",  2, { 0xFD, 0x4E, 0x00, 0x00 } },
  { "ld c, a",	1, { 0x4F, 0x00, 0x00, 0x00 } },

  { "ld d, b",	1, { 0x50, 0x00, 0x00, 0x00 } },
  { "ld d, c",	1, { 0x51, 0x00, 0x00, 0x00 } },
  { "ld d, d",	1, { 0x52, 0x00, 0x00, 0x00 } },
  { "ld d, e",	1, { 0x53, 0x00, 0x00, 0x00 } },
  { "ld d, h",	1, { 0x54, 0x00, 0x00, 0x00 } },
  { "ld d, l",	1, { 0x55, 0x00, 0x00, 0x00 } },
  { "ld d, (hl)",     1, { 0x56, 0x00, 0x00, 0x00 } },
  { "ld d, (ix+!d)",  2, { 0xDD, 0x56, 0x00, 0x00 } },
  { "ld d, (iy+!d)",  2, { 0xFD, 0x56, 0x00, 0x00 } },
  { "ld d, a",	1, { 0x57, 0x00, 0x00, 0x00 } },
  { "ld e, b",	1, { 0x58, 0x00, 0x00, 0x00 } },
  { "ld e, c",	1, { 0x59, 0x00, 0x00, 0x00 } },
  { "ld e, d",	1, { 0x5A, 0x00, 0x00, 0x00 } },
  { "ld e, e",	1, { 0x5B, 0x00, 0x00, 0x00 } },
  { "ld e, h",	1, { 0x5C, 0x00, 0x00, 0x00 } },
  { "ld e, l",	1, { 0x5D, 0x00, 0x00, 0x00 } },
  { "ld e, (hl)",     1, { 0x5E, 0x00, 0x00, 0x00 } },
  { "ld e, (ix+!d)",  2, { 0xDD, 0x5E, 0x00, 0x00 } },
  { "ld e, (iy+!d)",  2, { 0xFD, 0x5E, 0x00, 0x00 } },
  { "ld e, a",	1, { 0x5F, 0x00, 0x00, 0x00 } },

  { "ld h, b",	1, { 0x60, 0x00, 0x00, 0x00 } },
  { "ld h, c",	1, { 0x61, 0x00, 0x00, 0x00 } },
  { "ld h, d",	1, { 0x62, 0x00, 0x00, 0x00 } },
  { "ld h, e",	1, { 0x63, 0x00, 0x00, 0x00 } },
  { "ld h, h",	1, { 0x64, 0x00, 0x00, 0x00 } },
  { "ld h, l",	1, { 0x65, 0x00, 0x00, 0x00 } },
  { "ld h, (hl)",     1, { 0x66, 0x00, 0x00, 0x00 } },
  { "ld h, (ix+!d)",  2, { 0xDD, 0x66, 0x00, 0x00 } },
  { "ld h, (iy+!d)",  2, { 0xFD, 0x66, 0x00, 0x00 } },
  { "ld h, a",	1, { 0x67, 0x00, 0x00, 0x00 } },
  { "ld l, b",	1, { 0x68, 0x00, 0x00, 0x00 } },
  { "ld l, c",	1, { 0x69, 0x00, 0x00, 0x00 } },
  { "ld l, d",	1, { 0x6A, 0x00, 0x00, 0x00 } },
  { "ld l, e",	1, { 0x6B, 0x00, 0x00, 0x00 } },
  { "ld l, h",	1, { 0x6C, 0x00, 0x00, 0x00 } },
  { "ld l, l",	1, { 0x6D, 0x00, 0x00, 0x00 } },
  { "ld l, (hl)",     1, { 0x6E, 0x00, 0x00, 0x00 } },
  { "ld l, (ix+!d)",  2, { 0xDD, 0x6E, 0x00, 0x00 } },
  { "ld l, (iy+!d)",  2, { 0xFD, 0x6E, 0x00, 0x00 } },
  { "ld l, a",	1, { 0x6F, 0x00, 0x00, 0x00 } },

  { "ld (hl), b",     1, { 0x70, 0x00, 0x00, 0x00 } },
  { "ld (hl), c",     1, { 0x71, 0x00, 0x00, 0x00 } },
  { "ld (hl), d",     1, { 0x72, 0x00, 0x00, 0x00 } },
  { "ld (hl), e",     1, { 0x73, 0x00, 0x00, 0x00 } },
  { "ld (hl), h",     1, { 0x74, 0x00, 0x00, 0x00 } },
  { "ld (hl), l",     1, { 0x75, 0x00, 0x00, 0x00 } },
  { "ld (hl), a",     1, { 0x77, 0x00, 0x00, 0x00 } },
  { "ld a, b",	1, { 0x78, 0x00, 0x00, 0x00 } },
  { "ld a, c",	1, { 0x79, 0x00, 0x00, 0x00 } },
  { "ld a, d",	1, { 0x7A, 0x00, 0x00, 0x00 } },
  { "ld a, e",	1, { 0x7B, 0x00, 0x00, 0x00 } },
  { "ld a, h",	1, { 0x7C, 0x00, 0x00, 0x00 } },
  { "ld a, l",	1, { 0x7D, 0x00, 0x00, 0x00 } },
  { "ld a, (hl)",     1, { 0x7E, 0x00, 0x00, 0x00 } },
  { "ld a, (ix+!d)",  2, { 0xDD, 0x7E, 0x00, 0x00 } },
  { "ld a, (iy+!d)",  2, { 0xFD, 0x7E, 0x00, 0x00 } },
  { "ld a, a",	1, { 0x7F, 0x00, 0x00, 0x00 } },

  { "ld (ix+!d), b",  2, { 0xDD, 0x70, 0x00, 0x00 } },
  { "ld (ix+!d), c",  2, { 0xDD, 0x71, 0x00, 0x00 } },
  { "ld (ix+!d), d",  2, { 0xDD, 0x72, 0x00, 0x00 } },
  { "ld (ix+!d), e",  2, { 0xDD, 0x73, 0x00, 0x00 } },
  { "ld (ix+!d), h",  2, { 0xDD, 0x74, 0x00, 0x00 } },
  { "ld (ix+!d), l",  2, { 0xDD, 0x75, 0x00, 0x00 } },
  { "ld (ix+!d), a",  2, { 0xDD, 0x77, 0x00, 0x00 } },

  { "ld (iy+!d), b",  2, { 0xFD, 0x70, 0x00, 0x00 } },
  { "ld (iy+!d), c",  2, { 0xFD, 0x71, 0x00, 0x00 } },
  { "ld (iy+!d), d",  2, { 0xFD, 0x72, 0x00, 0x00 } },
  { "ld (iy+!d), e",  2, { 0xFD, 0x73, 0x00, 0x00 } },
  { "ld (iy+!d), h",  2, { 0xFD, 0x74, 0x00, 0x00 } },
  { "ld (iy+!d), l",  2, { 0xFD, 0x75, 0x00, 0x00 } },
  { "ld (iy+!d), a",  2, { 0xFD, 0x77, 0x00, 0x00 } },

  { "ld b, !1",       1, { 0x06, 0x00, 0x00, 0x00 } },
  { "ld c, !1",       1, { 0x0E, 0x00, 0x00, 0x00 } },
  { "ld d, !1",       1, { 0x16, 0x00, 0x00, 0x00 } },
  { "ld e, !1",       1, { 0x1E, 0x00, 0x00, 0x00 } },
  { "ld h, !1",       1, { 0x26, 0x00, 0x00, 0x00 } },
  { "ld l, !1",       1, { 0x2E, 0x00, 0x00, 0x00 } },
  { "ld (hl), !1",    1, { 0x36, 0x00, 0x00, 0x00 } },
  { "ld (ix+!d), !1", 2, { 0xDD, 0x36, 0x00, 0x00 } },
  { "ld (iy+!d), !1", 2, { 0xFD, 0x36, 0x00, 0x00 } },
  { "ld a, !1",       1, { 0x3E, 0x00, 0x00, 0x00 } },

  { "ld i, a",	2, { 0xED, 0x47, 0x00, 0x00 } },
  { "ld a, i",	2, { 0xED, 0x57, 0x00, 0x00 } },
  { "ld r, a",	2, { 0xED, 0x4F, 0x00, 0x00 } },
  { "ld a, r",	2, { 0xED, 0x57, 0x00, 0x00 } },

  { "ld sp, hl",      1, { 0xF9, 0x00, 0x00, 0x00 } },
  { "ld sp, ix",      2, { 0xDD, 0xF9, 0x00, 0x00 } },
  { "ld sp, iy",      2, { 0xFD, 0xF9, 0x00, 0x00 } },

  { "add hl, bc",     1, { 0x09, 0x00, 0x00, 0x00 } },
  { "add hl, de",     1, { 0x19, 0x00, 0x00, 0x00 } },
  { "add hl, hl",     1, { 0x29, 0x00, 0x00, 0x00 } },
  { "add hl, sp",     1, { 0x39, 0x00, 0x00, 0x00 } },

  { "sbc hl, bc",     2, { 0xED, 0x42, 0x00, 0x00 } },
  { "sbc hl, de",     2, { 0xED, 0x52, 0x00, 0x00 } },
  { "sbc hl, hl",     2, { 0xED, 0x62, 0x00, 0x00 } },
  { "sbc hl, sp",     2, { 0xED, 0x72, 0x00, 0x00 } },

  { "adc hl, bc",     2, { 0xED, 0x4A, 0x00, 0x00 } },
  { "adc hl, de",     2, { 0xED, 0x5A, 0x00, 0x00 } },
  { "adc hl, hl",     2, { 0xED, 0x6A, 0x00, 0x00 } },
  { "adc hl, sp",     2, { 0xED, 0x7A, 0x00, 0x00 } },

  { "add ix, bc",     2, { 0xDD, 0x09, 0x00, 0x00 } },
  { "add ix, de",     2, { 0xDD, 0x19, 0x00, 0x00 } },
  { "add ix, ix",     2, { 0xDD, 0x29, 0x00, 0x00 } },
  { "add ix, sp",     2, { 0xDD, 0x39, 0x00, 0x00 } },

  { "add iy, bc",     2, { 0xFD, 0x09, 0x00, 0x00 } },
  { "add iy, de",     2, { 0xFD, 0x19, 0x00, 0x00 } },
  { "add iy, iy",     2, { 0xFD, 0x29, 0x00, 0x00 } },
  { "add iy, sp",     2, { 0xFD, 0x39, 0x00, 0x00 } },

  { "add a, b",       1, { 0x80, 0x00, 0x00, 0x00 } },
  { "add a, c",       1, { 0x81, 0x00, 0x00, 0x00 } },
  { "add a, d",       1, { 0x82, 0x00, 0x00, 0x00 } },
  { "add a, e",       1, { 0x83, 0x00, 0x00, 0x00 } },
  { "add a, h",       1, { 0x84, 0x00, 0x00, 0x00 } },
  { "add a, l",       1, { 0x85, 0x00, 0x00, 0x00 } },
  { "add a, (hl)",    1, { 0x86, 0x00, 0x00, 0x00 } },
  { "add a, (ix+!d)", 2, { 0xDD, 0x86, 0x00, 0x00 } },
  { "add a, (iy+!d)", 2, { 0xFD, 0x86, 0x00, 0x00 } },
  { "add a, a",       1, { 0x87, 0x00, 0x00, 0x00 } },
  { "add a, !1",      1, { 0xC6, 0x00, 0x00, 0x00 } },
  { "adc a, b",       1, { 0x88, 0x00, 0x00, 0x00 } },
  { "adc a, c",       1, { 0x89, 0x00, 0x00, 0x00 } },
  { "adc a, d",       1, { 0x8A, 0x00, 0x00, 0x00 } },
  { "adc a, e",       1, { 0x8B, 0x00, 0x00, 0x00 } },
  { "adc a, h",       1, { 0x8C, 0x00, 0x00, 0x00 } },
  { "adc a, l",       1, { 0x8D, 0x00, 0x00, 0x00 } },
  { "adc a, (hl)",    1, { 0x8E, 0x00, 0x00, 0x00 } },
  { "adc a, (ix+!d)", 2, { 0xDD, 0x8E, 0x00, 0x00 } },
  { "adc a, (iy+!d)", 2, { 0xFD, 0x8E, 0x00, 0x00 } },
  { "adc a, a",       1, { 0x8F, 0x00, 0x00, 0x00 } },
  { "adc a, !1",      1, { 0xCE, 0x00, 0x00, 0x00 } },

  { "sub a, b",       1, { 0x90, 0x00, 0x00, 0x00 } },
  { "sub a, c",       1, { 0x91, 0x00, 0x00, 0x00 } },
  { "sub a, d",       1, { 0x92, 0x00, 0x00, 0x00 } },
  { "sub a, e",       1, { 0x93, 0x00, 0x00, 0x00 } },
  { "sub a, h",       1, { 0x94, 0x00, 0x00, 0x00 } },
  { "sub a, l",       1, { 0x95, 0x00, 0x00, 0x00 } },
  { "sub a, (hl)",    1, { 0x96, 0x00, 0x00, 0x00 } },
  { "sub a, (ix+!d)", 2, { 0xDD, 0x96, 0x00, 0x00 } },
  { "sub a, (iy+!d)", 2, { 0xFD, 0x96, 0x00, 0x00 } },
  { "sub a, a",       1, { 0x97, 0x00, 0x00, 0x00 } },
  { "sub a, !1",      1, { 0xD6, 0x00, 0x00, 0x00 } },
  { "sbc a, b",       1, { 0x98, 0x00, 0x00, 0x00 } },
  { "sbc a, c",       1, { 0x99, 0x00, 0x00, 0x00 } },
  { "sbc a, d",       1, { 0x9A, 0x00, 0x00, 0x00 } },
  { "sbc a, e",       1, { 0x9B, 0x00, 0x00, 0x00 } },
  { "sbc a, h",       1, { 0x9C, 0x00, 0x00, 0x00 } },
  { "sbc a, l",       1, { 0x9D, 0x00, 0x00, 0x00 } },
  { "sbc a, (hl)",    1, { 0x9E, 0x00, 0x00, 0x00 } },
  { "sbc a, (ix+!d)", 2, { 0xDD, 0x9E, 0x00, 0x00 } },
  { "sbc a, (iy+!d)", 2, { 0xFD, 0x9E, 0x00, 0x00 } },
  { "sbc a, a",       1, { 0x9F, 0x00, 0x00, 0x00 } },
  { "sbc a, !1",      1, { 0xDE, 0x00, 0x00, 0x00 } },

  { "and a, b",       1, { 0xA0, 0x00, 0x00, 0x00 } },
  { "and a, c",       1, { 0xA1, 0x00, 0x00, 0x00 } },
  { "and a, d",       1, { 0xA2, 0x00, 0x00, 0x00 } },
  { "and a, e",       1, { 0xA3, 0x00, 0x00, 0x00 } },
  { "and a, h",       1, { 0xA4, 0x00, 0x00, 0x00 } },
  { "and a, l",       1, { 0xA5, 0x00, 0x00, 0x00 } },
  { "and a, (hl)",    1, { 0xA6, 0x00, 0x00, 0x00 } },
  { "and a, (ix+!d)", 2, { 0xDD, 0xA6, 0x00, 0x00 } },
  { "and a, (iy+!d)", 2, { 0xFD, 0xA6, 0x00, 0x00 } },
  { "and a, a",       1, { 0xA7, 0x00, 0x00, 0x00 } },
  { "and a, !1",      1, { 0xE6, 0x00, 0x00, 0x00 } },
  { "xor a, b",       1, { 0xA8, 0x00, 0x00, 0x00 } },
  { "xor a, c",       1, { 0xA9, 0x00, 0x00, 0x00 } },
  { "xor a, d",       1, { 0xAA, 0x00, 0x00, 0x00 } },
  { "xor a, e",       1, { 0xAB, 0x00, 0x00, 0x00 } },
  { "xor a, h",       1, { 0xAC, 0x00, 0x00, 0x00 } },
  { "xor a, l",       1, { 0xAD, 0x00, 0x00, 0x00 } },
  { "xor a, (hl)",    1, { 0xAE, 0x00, 0x00, 0x00 } },
  { "xor a, (ix+!d)", 2, { 0xDD, 0xAE, 0x00, 0x00 } },
  { "xor a, (iy+!d)", 2, { 0xFD, 0xAE, 0x00, 0x00 } },
  { "xor a, a",       1, { 0xAF, 0x00, 0x00, 0x00 } },
  { "xor a, !1",      1, { 0xEE, 0x00, 0x00, 0x00 } },

  { "or a, b",       1, { 0xB0, 0x00, 0x00, 0x00 } },
  { "or a, c",       1, { 0xB1, 0x00, 0x00, 0x00 } },
  { "or a, d",       1, { 0xB2, 0x00, 0x00, 0x00 } },
  { "or a, e",       1, { 0xB3, 0x00, 0x00, 0x00 } },
  { "or a, h",       1, { 0xB4, 0x00, 0x00, 0x00 } },
  { "or a, l",       1, { 0xB5, 0x00, 0x00, 0x00 } },
  { "or a, (hl)",    1, { 0xB6, 0x00, 0x00, 0x00 } },
  { "or a, (ix+!d)", 2, { 0xDD, 0xB6, 0x00, 0x00 } },
  { "or a, (iy+!d)", 2, { 0xFD, 0xB6, 0x00, 0x00 } },
  { "or a, a",       1, { 0xB7, 0x00, 0x00, 0x00 } },
  { "or a, !1",      1, { 0xF6, 0x00, 0x00, 0x00 } },
  { "cp a, b",       1, { 0xB8, 0x00, 0x00, 0x00 } },
  { "cp a, c",       1, { 0xB9, 0x00, 0x00, 0x00 } },
  { "cp a, d",       1, { 0xBA, 0x00, 0x00, 0x00 } },
  { "cp a, e",       1, { 0xBB, 0x00, 0x00, 0x00 } },
  { "cp a, h",       1, { 0xBC, 0x00, 0x00, 0x00 } },
  { "cp a, l",       1, { 0xBD, 0x00, 0x00, 0x00 } },
  { "cp a, (hl)",    1, { 0xBE, 0x00, 0x00, 0x00 } },
  { "cp a, (ix+!d)", 2, { 0xDD, 0xBE, 0x00, 0x00 } },
  { "cp a, (iy+!d)", 2, { 0xFD, 0xBE, 0x00, 0x00 } },
  { "cp a, a",       1, { 0xBF, 0x00, 0x00, 0x00 } },
  { "cp a, !1",      1, { 0xFE, 0x00, 0x00, 0x00 } },

  { "jr nz, !r",      1, { 0x20, 0x00, 0x00, 0x00 } },
  { "jr z, !r",       1, { 0x28, 0x00, 0x00, 0x00 } },
  { "jr nc, !r",      1, { 0x30, 0x00, 0x00, 0x00 } },
  { "jr c, !r",       1, { 0x38, 0x00, 0x00, 0x00 } },
  { "djnz !r",	1, { 0x10, 0x00, 0x00, 0x00 } },
  { "jr !r",	  1, { 0x18, 0x00, 0x00, 0x00 } },

  { "jp nz, !2",      1, { 0xC2, 0x00, 0x00, 0x00 } },
  { "jp nc, !2",      1, { 0xD2, 0x00, 0x00, 0x00 } },
  { "jp po, !2",      1, { 0xE2, 0x00, 0x00, 0x00 } },
  { "jp p, !2",       1, { 0xF2, 0x00, 0x00, 0x00 } },
  { "jp z, !2",       1, { 0xCA, 0x00, 0x00, 0x00 } },
  { "jp c, !2",       1, { 0xDA, 0x00, 0x00, 0x00 } },
  { "jp pe, !2",      1, { 0xEA, 0x00, 0x00, 0x00 } },
  { "jp n, !2",       1, { 0xFA, 0x00, 0x00, 0x00 } },
  { "jp !2",	  1, { 0xC3, 0x00, 0x00, 0x00 } },

  { "call nz, !2",    1, { 0xC4, 0x00, 0x00, 0x00 } },
  { "call nc, !2",    1, { 0xD4, 0x00, 0x00, 0x00 } },
  { "call po, !2",    1, { 0xE4, 0x00, 0x00, 0x00 } },
  { "call p, !2",     1, { 0xF4, 0x00, 0x00, 0x00 } },
  { "call z, !2",     1, { 0xCC, 0x00, 0x00, 0x00 } },
  { "call c, !2",     1, { 0xDC, 0x00, 0x00, 0x00 } },
  { "call pe, !2",    1, { 0xEC, 0x00, 0x00, 0x00 } },
  { "call n, !2",     1, { 0xFC, 0x00, 0x00, 0x00 } },
  { "call !2",	1, { 0xCD, 0x00, 0x00, 0x00 } },

  { "in a, (!1)",     1, { 0xDB, 0x00, 0x00, 0x00 } },
  { "in b, (c)",      2, { 0xED, 0x40, 0x00, 0x00 } },
  { "in c, (c)",      2, { 0xED, 0x48, 0x00, 0x00 } },
  { "in d, (c)",      2, { 0xED, 0x50, 0x00, 0x00 } },
  { "in e, (c)",      2, { 0xED, 0x58, 0x00, 0x00 } },
  { "in h, (c)",      2, { 0xED, 0x60, 0x00, 0x00 } },
  { "in l, (c)",      2, { 0xED, 0x68, 0x00, 0x00 } },
  { "in f, (c)",      2, { 0xED, 0x70, 0x00, 0x00 } },
  { "in a, (c)",      2, { 0xED, 0x78, 0x00, 0x00 } },

  { "out (!1), a",    1, { 0xD3, 0x00, 0x00, 0x00 } },
  { "out (c), b",     2, { 0xED, 0x41, 0x00, 0x00 } },
  { "out (c), c",     2, { 0xED, 0x49, 0x00, 0x00 } },
  { "out (c), d",     2, { 0xED, 0x51, 0x00, 0x00 } },
  { "out (c), e",     2, { 0xED, 0x59, 0x00, 0x00 } },
  { "out (c), h",     2, { 0xED, 0x61, 0x00, 0x00 } },
  { "out (c), l",     2, { 0xED, 0x69, 0x00, 0x00 } },
  { "out (c), 0",     2, { 0xED, 0x71, 0x00, 0x00 } },
  { "out (c), a",     2, { 0xED, 0x79, 0x00, 0x00 } },

  { "rlc b",	  2, { 0xCB, 0x00, 0x00, 0x00 } },
  { "rlc c",	  2, { 0xCB, 0x01, 0x00, 0x00 } },
  { "rlc d",	  2, { 0xCB, 0x02, 0x00, 0x00 } },
  { "rlc e",	  2, { 0xCB, 0x03, 0x00, 0x00 } },
  { "rlc h",	  2, { 0xCB, 0x04, 0x00, 0x00 } },
  { "rlc l",	  2, { 0xCB, 0x05, 0x00, 0x00 } },
  { "rlc (hl)",       2, { 0xCB, 0x06, 0x00, 0x00 } },
  { "rlc (ix+!b)",    3, { 0xDD, 0xCB, 0x06, 0x00 } },
  { "rlc (iy+!b)",    3, { 0xFD, 0xCB, 0x06, 0x00 } },
  { "rlc a",	  2, { 0xCB, 0x07, 0x00, 0x00 } },
  { "rrc b",	  2, { 0xCB, 0x08, 0x00, 0x00 } },
  { "rrc c",	  2, { 0xCB, 0x09, 0x00, 0x00 } },
  { "rrc d",	  2, { 0xCB, 0x0A, 0x00, 0x00 } },
  { "rrc e",	  2, { 0xCB, 0x0B, 0x00, 0x00 } },
  { "rrc h",	  2, { 0xCB, 0x0C, 0x00, 0x00 } },
  { "rrc l",	  2, { 0xCB, 0x0D, 0x00, 0x00 } },
  { "rrc (hl)",       2, { 0xCB, 0x0E, 0x00, 0x00 } },
  { "rrc (ix+!b)",    3, { 0xDD, 0xCB, 0x0E, 0x00 } },
  { "rrc (iy+!b)",    3, { 0xFD, 0xCB, 0x0E, 0x00 } },
  { "rrc a",	  2, { 0xCB, 0x0F, 0x00, 0x00 } },

  { "rl b",	   2, { 0xCB, 0x10, 0x00, 0x00 } },
  { "rl c",	   2, { 0xCB, 0x11, 0x00, 0x00 } },
  { "rl d",	   2, { 0xCB, 0x12, 0x00, 0x00 } },
  { "rl e",	   2, { 0xCB, 0x13, 0x00, 0x00 } },
  { "rl h",	   2, { 0xCB, 0x14, 0x00, 0x00 } },
  { "rl l",	   2, { 0xCB, 0x15, 0x00, 0x00 } },
  { "rl (hl)",	2, { 0xCB, 0x16, 0x00, 0x00 } },
  { "rl (ix+!b)",     3, { 0xDD, 0xCB, 0x16, 0x00 } },
  { "rl (iy+!b)",     3, { 0xFD, 0xCB, 0x16, 0x00 } },
  { "rl a",	   2, { 0xCB, 0x17, 0x00, 0x00 } },
  { "rr b",	   2, { 0xCB, 0x18, 0x00, 0x00 } },
  { "rr c",	   2, { 0xCB, 0x19, 0x00, 0x00 } },
  { "rr d",	   2, { 0xCB, 0x1A, 0x00, 0x00 } },
  { "rr e",	   2, { 0xCB, 0x1B, 0x00, 0x00 } },
  { "rr h",	   2, { 0xCB, 0x1C, 0x00, 0x00 } },
  { "rr l",	   2, { 0xCB, 0x1D, 0x00, 0x00 } },
  { "rr (hl)",	2, { 0xCB, 0x1E, 0x00, 0x00 } },
  { "rr (ix+!b)",     3, { 0xDD, 0xCB, 0x1E, 0x00 } },
  { "rr (iy+!b)",     3, { 0xFD, 0xCB, 0x1E, 0x00 } },
  { "rr a",	   2, { 0xCB, 0x1F, 0x00, 0x00 } },

  { "sla b",	  2, { 0xCB, 0x20, 0x00, 0x00 } },
  { "sla c",	  2, { 0xCB, 0x21, 0x00, 0x00 } },
  { "sla d",	  2, { 0xCB, 0x22, 0x00, 0x00 } },
  { "sla e",	  2, { 0xCB, 0x23, 0x00, 0x00 } },
  { "sla h",	  2, { 0xCB, 0x24, 0x00, 0x00 } },
  { "sla l",	  2, { 0xCB, 0x25, 0x00, 0x00 } },
  { "sla (hl)",       2, { 0xCB, 0x26, 0x00, 0x00 } },
  { "sla (ix+!b)",    3, { 0xDD, 0xCB, 0x26, 0x00 } },
  { "sla (iy+!b)",    3, { 0xFD, 0xCB, 0x26, 0x00 } },
  { "sla a",	  2, { 0xCB, 0x27, 0x00, 0x00 } },
  { "sra b",	  2, { 0xCB, 0x28, 0x00, 0x00 } },
  { "sra c",	  2, { 0xCB, 0x29, 0x00, 0x00 } },
  { "sra d",	  2, { 0xCB, 0x2A, 0x00, 0x00 } },
  { "sra e",	  2, { 0xCB, 0x2B, 0x00, 0x00 } },
  { "sra h",	  2, { 0xCB, 0x2C, 0x00, 0x00 } },
  { "sra l",	  2, { 0xCB, 0x2D, 0x00, 0x00 } },
  { "sra (hl)",       2, { 0xCB, 0x2E, 0x00, 0x00 } },
  { "sra (ix+!b)",    3, { 0xDD, 0xCB, 0x2E, 0x00 } },
  { "sra (iy+!b)",    3, { 0xFD, 0xCB, 0x2E, 0x00 } },
  { "sra a",	  2, { 0xCB, 0x2F, 0x00, 0x00 } },

  { "sll b",	  2, { 0xCB, 0x30, 0x00, 0x00 } },
  { "sll c",	  2, { 0xCB, 0x31, 0x00, 0x00 } },
  { "sll d",	  2, { 0xCB, 0x32, 0x00, 0x00 } },
  { "sll e",	  2, { 0xCB, 0x33, 0x00, 0x00 } },
  { "sll h",	  2, { 0xCB, 0x34, 0x00, 0x00 } },
  { "sll l",	  2, { 0xCB, 0x35, 0x00, 0x00 } },
  { "sll (hl)",       2, { 0xCB, 0x36, 0x00, 0x00 } },
  { "sll (ix+!b)",    3, { 0xDD, 0xCB, 0x36, 0x00 } },
  { "sll (iy+!b)",    3, { 0xFD, 0xCB, 0x36, 0x00 } },
  { "sll a",	  2, { 0xCB, 0x37, 0x00, 0x00 } },
  { "srl b",	  2, { 0xCB, 0x38, 0x00, 0x00 } },
  { "srl c",	  2, { 0xCB, 0x39, 0x00, 0x00 } },
  { "srl d",	  2, { 0xCB, 0x3A, 0x00, 0x00 } },
  { "srl e",	  2, { 0xCB, 0x3B, 0x00, 0x00 } },
  { "srl h",	  2, { 0xCB, 0x3C, 0x00, 0x00 } },
  { "srl l",	  2, { 0xCB, 0x3D, 0x00, 0x00 } },
  { "srl (hl)",       2, { 0xCB, 0x3E, 0x00, 0x00 } },
  { "srl (ix+!b)",    3, { 0xDD, 0xCB, 0x3E, 0x00 } },
  { "srl (iy+!b)",    3, { 0xFD, 0xCB, 0x3E, 0x00 } },
  { "srl a",	  2, { 0xCB, 0x3F, 0x00, 0x00 } },

  { "bit 0, b",       2, { 0xCB, 0x40, 0x00, 0x00 } },
  { "bit 0, c",       2, { 0xCB, 0x41, 0x00, 0x00 } },
  { "bit 0, d",       2, { 0xCB, 0x42, 0x00, 0x00 } },
  { "bit 0, e",       2, { 0xCB, 0x43, 0x00, 0x00 } },
  { "bit 0, h",       2, { 0xCB, 0x44, 0x00, 0x00 } },
  { "bit 0, l",       2, { 0xCB, 0x45, 0x00, 0x00 } },
  { "bit 0, (hl)",    2, { 0xCB, 0x46, 0x00, 0x00 } },
  { "bit 0, (ix+!b)", 3, { 0xDD, 0xCB, 0x46, 0x00 } },
  { "bit 0, (iy+!b)", 3, { 0xFD, 0xCB, 0x46, 0x00 } },
  { "bit 0, a",       2, { 0xCB, 0x47, 0x00, 0x00 } },
  { "bit 1, b",       2, { 0xCB, 0x48, 0x00, 0x00 } },
  { "bit 1, c",       2, { 0xCB, 0x49, 0x00, 0x00 } },
  { "bit 1, d",       2, { 0xCB, 0x4A, 0x00, 0x00 } },
  { "bit 1, e",       2, { 0xCB, 0x4B, 0x00, 0x00 } },
  { "bit 1, h",       2, { 0xCB, 0x4C, 0x00, 0x00 } },
  { "bit 1, l",       2, { 0xCB, 0x4D, 0x00, 0x00 } },
  { "bit 1, (hl)",    2, { 0xCB, 0x4E, 0x00, 0x00 } },
  { "bit 1, (ix+!b)", 3, { 0xDD, 0xCB, 0x4E, 0x00 } },
  { "bit 1, (iy+!b)", 3, { 0xFD, 0xCB, 0x4E, 0x00 } },
  { "bit 1, a",       2, { 0xCB, 0x4F, 0x00, 0x00 } },
  { "bit 2, b",       2, { 0xCB, 0x50, 0x00, 0x00 } },
  { "bit 2, c",       2, { 0xCB, 0x51, 0x00, 0x00 } },
  { "bit 2, d",       2, { 0xCB, 0x52, 0x00, 0x00 } },
  { "bit 2, e",       2, { 0xCB, 0x53, 0x00, 0x00 } },
  { "bit 2, h",       2, { 0xCB, 0x54, 0x00, 0x00 } },
  { "bit 2, l",       2, { 0xCB, 0x55, 0x00, 0x00 } },
  { "bit 2, (hl)",    2, { 0xCB, 0x56, 0x00, 0x00 } },
  { "bit 2, (ix+!b)", 3, { 0xDD, 0xCB, 0x56, 0x00 } },
  { "bit 2, (iy+!b)", 3, { 0xFD, 0xCB, 0x56, 0x00 } },
  { "bit 2, a",       2, { 0xCB, 0x57, 0x00, 0x00 } },
  { "bit 3, b",       2, { 0xCB, 0x58, 0x00, 0x00 } },
  { "bit 3, c",       2, { 0xCB, 0x59, 0x00, 0x00 } },
  { "bit 3, d",       2, { 0xCB, 0x5A, 0x00, 0x00 } },
  { "bit 3, e",       2, { 0xCB, 0x5B, 0x00, 0x00 } },
  { "bit 3, h",       2, { 0xCB, 0x5C, 0x00, 0x00 } },
  { "bit 3, l",       2, { 0xCB, 0x5D, 0x00, 0x00 } },
  { "bit 3, (hl)",    2, { 0xCB, 0x5E, 0x00, 0x00 } },
  { "bit 3, (ix+!b)", 3, { 0xDD, 0xCB, 0x5E, 0x00 } },
  { "bit 3, (iy+!b)", 3, { 0xFD, 0xCB, 0x5E, 0x00 } },
  { "bit 3, a",       2, { 0xCB, 0x5F, 0x00, 0x00 } },
  { "bit 4, b",       2, { 0xCB, 0x60, 0x00, 0x00 } },
  { "bit 4, c",       2, { 0xCB, 0x61, 0x00, 0x00 } },
  { "bit 4, d",       2, { 0xCB, 0x62, 0x00, 0x00 } },
  { "bit 4, e",       2, { 0xCB, 0x63, 0x00, 0x00 } },
  { "bit 4, h",       2, { 0xCB, 0x64, 0x00, 0x00 } },
  { "bit 4, l",       2, { 0xCB, 0x65, 0x00, 0x00 } },
  { "bit 4, (hl)",    2, { 0xCB, 0x66, 0x00, 0x00 } },
  { "bit 4, (ix+!b)", 3, { 0xDD, 0xCB, 0x66, 0x00 } },
  { "bit 4, (iy+!b)", 3, { 0xFD, 0xCB, 0x66, 0x00 } },
  { "bit 4, a",       2, { 0xCB, 0x67, 0x00, 0x00 } },
  { "bit 5, b",       2, { 0xCB, 0x68, 0x00, 0x00 } },
  { "bit 5, c",       2, { 0xCB, 0x69, 0x00, 0x00 } },
  { "bit 5, d",       2, { 0xCB, 0x6A, 0x00, 0x00 } },
  { "bit 5, e",       2, { 0xCB, 0x6B, 0x00, 0x00 } },
  { "bit 5, h",       2, { 0xCB, 0x6C, 0x00, 0x00 } },
  { "bit 5, l",       2, { 0xCB, 0x6D, 0x00, 0x00 } },
  { "bit 5, (hl)",    2, { 0xCB, 0x6E, 0x00, 0x00 } },
  { "bit 5, (ix+!b)", 3, { 0xDD, 0xCB, 0x6E, 0x00 } },
  { "bit 5, (iy+!b)", 3, { 0xFD, 0xCB, 0x6E, 0x00 } },
  { "bit 5, a",       2, { 0xCB, 0x6F, 0x00, 0x00 } },
  { "bit 6, b",       2, { 0xCB, 0x70, 0x00, 0x00 } },
  { "bit 6, c",       2, { 0xCB, 0x71, 0x00, 0x00 } },
  { "bit 6, d",       2, { 0xCB, 0x72, 0x00, 0x00 } },
  { "bit 6, e",       2, { 0xCB, 0x73, 0x00, 0x00 } },
  { "bit 6, h",       2, { 0xCB, 0x74, 0x00, 0x00 } },
  { "bit 6, l",       2, { 0xCB, 0x75, 0x00, 0x00 } },
  { "bit 6, (hl)",    2, { 0xCB, 0x76, 0x00, 0x00 } },
  { "bit 6, (ix+!b)", 3, { 0xDD, 0xCB, 0x76, 0x00 } },
  { "bit 6, (iy+!b)", 3, { 0xFD, 0xCB, 0x76, 0x00 } },
  { "bit 6, a",       2, { 0xCB, 0x77, 0x00, 0x00 } },
  { "bit 7, b",       2, { 0xCB, 0x78, 0x00, 0x00 } },
  { "bit 7, c",       2, { 0xCB, 0x79, 0x00, 0x00 } },
  { "bit 7, d",       2, { 0xCB, 0x7A, 0x00, 0x00 } },
  { "bit 7, e",       2, { 0xCB, 0x7B, 0x00, 0x00 } },
  { "bit 7, h",       2, { 0xCB, 0x7C, 0x00, 0x00 } },
  { "bit 7, l",       2, { 0xCB, 0x7D, 0x00, 0x00 } },
  { "bit 7, (hl)",    2, { 0xCB, 0x7E, 0x00, 0x00 } },
  { "bit 7, (ix+!b)", 3, { 0xDD, 0xCB, 0x7E, 0x00 } },
  { "bit 7, (iy+!b)", 3, { 0xFD, 0xCB, 0x7E, 0x00 } },
  { "bit 7, a",       2, { 0xCB, 0x7F, 0x00, 0x00 } },

  { "res 0, b",       2, { 0xCB, 0x80, 0x00, 0x00 } },
  { "res 0, c",       2, { 0xCB, 0x81, 0x00, 0x00 } },
  { "res 0, d",       2, { 0xCB, 0x82, 0x00, 0x00 } },
  { "res 0, e",       2, { 0xCB, 0x83, 0x00, 0x00 } },
  { "res 0, h",       2, { 0xCB, 0x84, 0x00, 0x00 } },
  { "res 0, l",       2, { 0xCB, 0x85, 0x00, 0x00 } },
  { "res 0, (hl)",    2, { 0xCB, 0x86, 0x00, 0x00 } },
  { "res 0, (ix+!b)", 3, { 0xDD, 0xCB, 0x86, 0x00 } },
  { "res 0, (iy+!b)", 3, { 0xFD, 0xCB, 0x86, 0x00 } },
  { "res 0, a",       2, { 0xCB, 0x87, 0x00, 0x00 } },
  { "res 1, b",       2, { 0xCB, 0x88, 0x00, 0x00 } },
  { "res 1, c",       2, { 0xCB, 0x89, 0x00, 0x00 } },
  { "res 1, d",       2, { 0xCB, 0x8A, 0x00, 0x00 } },
  { "res 1, e",       2, { 0xCB, 0x8B, 0x00, 0x00 } },
  { "res 1, h",       2, { 0xCB, 0x8C, 0x00, 0x00 } },
  { "res 1, l",       2, { 0xCB, 0x8D, 0x00, 0x00 } },
  { "res 1, (hl)",    2, { 0xCB, 0x8E, 0x00, 0x00 } },
  { "res 1, (ix+!b)", 3, { 0xDD, 0xCB, 0x8E, 0x00 } },
  { "res 1, (iy+!b)", 3, { 0xFD, 0xCB, 0x8E, 0x00 } },
  { "res 1, a",       2, { 0xCB, 0x8F, 0x00, 0x00 } },
  { "res 2, b",       2, { 0xCB, 0x90, 0x00, 0x00 } },
  { "res 2, c",       2, { 0xCB, 0x91, 0x00, 0x00 } },
  { "res 2, d",       2, { 0xCB, 0x92, 0x00, 0x00 } },
  { "res 2, e",       2, { 0xCB, 0x93, 0x00, 0x00 } },
  { "res 2, h",       2, { 0xCB, 0x94, 0x00, 0x00 } },
  { "res 2, l",       2, { 0xCB, 0x95, 0x00, 0x00 } },
  { "res 2, (hl)",    2, { 0xCB, 0x96, 0x00, 0x00 } },
  { "res 2, (ix+!b)", 3, { 0xDD, 0xCB, 0x96, 0x00 } },
  { "res 2, (iy+!b)", 3, { 0xFD, 0xCB, 0x96, 0x00 } },
  { "res 2, a",       2, { 0xCB, 0x97, 0x00, 0x00 } },
  { "res 3, b",       2, { 0xCB, 0x98, 0x00, 0x00 } },
  { "res 3, c",       2, { 0xCB, 0x99, 0x00, 0x00 } },
  { "res 3, d",       2, { 0xCB, 0x9A, 0x00, 0x00 } },
  { "res 3, e",       2, { 0xCB, 0x9B, 0x00, 0x00 } },
  { "res 3, h",       2, { 0xCB, 0x9C, 0x00, 0x00 } },
  { "res 3, l",       2, { 0xCB, 0x9D, 0x00, 0x00 } },
  { "res 3, (hl)",    2, { 0xCB, 0x9E, 0x00, 0x00 } },
  { "res 3, (ix+!b)", 3, { 0xDD, 0xCB, 0x9E, 0x00 } },
  { "res 3, (iy+!b)", 3, { 0xFD, 0xCB, 0x9E, 0x00 } },
  { "res 3, a",       2, { 0xCB, 0x9F, 0x00, 0x00 } },
  { "res 4, b",       2, { 0xCB, 0xA0, 0x00, 0x00 } },
  { "res 4, c",       2, { 0xCB, 0xA1, 0x00, 0x00 } },
  { "res 4, d",       2, { 0xCB, 0xA2, 0x00, 0x00 } },
  { "res 4, e",       2, { 0xCB, 0xA3, 0x00, 0x00 } },
  { "res 4, h",       2, { 0xCB, 0xA4, 0x00, 0x00 } },
  { "res 4, l",       2, { 0xCB, 0xA5, 0x00, 0x00 } },
  { "res 4, (hl)",    2, { 0xCB, 0xA6, 0x00, 0x00 } },
  { "res 4, (ix+!b)", 3, { 0xDD, 0xCB, 0xA6, 0x00 } },
  { "res 4, (iy+!b)", 3, { 0xFD, 0xCB, 0xA6, 0x00 } },
  { "res 4, a",       2, { 0xCB, 0xA7, 0x00, 0x00 } },
  { "res 5, b",       2, { 0xCB, 0xA8, 0x00, 0x00 } },
  { "res 5, c",       2, { 0xCB, 0xA9, 0x00, 0x00 } },
  { "res 5, d",       2, { 0xCB, 0xAA, 0x00, 0x00 } },
  { "res 5, e",       2, { 0xCB, 0xAB, 0x00, 0x00 } },
  { "res 5, h",       2, { 0xCB, 0xAC, 0x00, 0x00 } },
  { "res 5, l",       2, { 0xCB, 0xAD, 0x00, 0x00 } },
  { "res 5, (hl)",    2, { 0xCB, 0xAE, 0x00, 0x00 } },
  { "res 5, (ix+!b)", 3, { 0xDD, 0xCB, 0xAE, 0x00 } },
  { "res 5, (iy+!b)", 3, { 0xFD, 0xCB, 0xAE, 0x00 } },
  { "res 5, a",       2, { 0xCB, 0xAF, 0x00, 0x00 } },
  { "res 6, b",       2, { 0xCB, 0xB0, 0x00, 0x00 } },
  { "res 6, c",       2, { 0xCB, 0xB1, 0x00, 0x00 } },
  { "res 6, d",       2, { 0xCB, 0xB2, 0x00, 0x00 } },
  { "res 6, e",       2, { 0xCB, 0xB3, 0x00, 0x00 } },
  { "res 6, h",       2, { 0xCB, 0xB4, 0x00, 0x00 } },
  { "res 6, l",       2, { 0xCB, 0xB5, 0x00, 0x00 } },
  { "res 6, (hl)",    2, { 0xCB, 0xB6, 0x00, 0x00 } },
  { "res 6, (ix+!b)", 3, { 0xDD, 0xCB, 0xB6, 0x00 } },
  { "res 6, (iy+!b)", 3, { 0xFD, 0xCB, 0xB6, 0x00 } },
  { "res 6, a",       2, { 0xCB, 0xB7, 0x00, 0x00 } },
  { "res 7, b",       2, { 0xCB, 0xB8, 0x00, 0x00 } },
  { "res 7, c",       2, { 0xCB, 0xB9, 0x00, 0x00 } },
  { "res 7, d",       2, { 0xCB, 0xBA, 0x00, 0x00 } },
  { "res 7, e",       2, { 0xCB, 0xBB, 0x00, 0x00 } },
  { "res 7, h",       2, { 0xCB, 0xBC, 0x00, 0x00 } },
  { "res 7, l",       2, { 0xCB, 0xBD, 0x00, 0x00 } },
  { "res 7, (hl)",    2, { 0xCB, 0xBE, 0x00, 0x00 } },
  { "res 7, (ix+!b)", 3, { 0xDD, 0xCB, 0xBE, 0x00 } },
  { "res 7, (iy+!b)", 3, { 0xFD, 0xCB, 0xBE, 0x00 } },
  { "res 7, a",       2, { 0xCB, 0xBF, 0x00, 0x00 } },

  { "set 0, b",       2, { 0xCB, 0xC0, 0x00, 0x00 } },
  { "set 0, c",       2, { 0xCB, 0xC1, 0x00, 0x00 } },
  { "set 0, d",       2, { 0xCB, 0xC2, 0x00, 0x00 } },
  { "set 0, e",       2, { 0xCB, 0xC3, 0x00, 0x00 } },
  { "set 0, h",       2, { 0xCB, 0xC4, 0x00, 0x00 } },
  { "set 0, l",       2, { 0xCB, 0xC5, 0x00, 0x00 } },
  { "set 0, (hl)",    2, { 0xCB, 0xC6, 0x00, 0x00 } },
  { "set 0, (ix+!b)", 3, { 0xDD, 0xCB, 0xC6, 0x00 } },
  { "set 0, (iy+!b)", 3, { 0xFD, 0xCB, 0xC6, 0x00 } },
  { "set 0, a",       2, { 0xCB, 0xC7, 0x00, 0x00 } },
  { "set 1, b",       2, { 0xCB, 0xC8, 0x00, 0x00 } },
  { "set 1, c",       2, { 0xCB, 0xC9, 0x00, 0x00 } },
  { "set 1, d",       2, { 0xCB, 0xCA, 0x00, 0x00 } },
  { "set 1, e",       2, { 0xCB, 0xCB, 0x00, 0x00 } },
  { "set 1, h",       2, { 0xCB, 0xCC, 0x00, 0x00 } },
  { "set 1, l",       2, { 0xCB, 0xCD, 0x00, 0x00 } },
  { "set 1, (hl)",    2, { 0xCB, 0xCE, 0x00, 0x00 } },
  { "set 1, (ix+!b)", 3, { 0xDD, 0xCB, 0xCE, 0x00 } },
  { "set 1, (iy+!b)", 3, { 0xFD, 0xCB, 0xCE, 0x00 } },
  { "set 1, a",       2, { 0xCB, 0xCF, 0x00, 0x00 } },
  { "set 2, b",       2, { 0xCB, 0xD0, 0x00, 0x00 } },
  { "set 2, c",       2, { 0xCB, 0xD1, 0x00, 0x00 } },
  { "set 2, d",       2, { 0xCB, 0xD2, 0x00, 0x00 } },
  { "set 2, e",       2, { 0xCB, 0xD3, 0x00, 0x00 } },
  { "set 2, h",       2, { 0xCB, 0xD4, 0x00, 0x00 } },
  { "set 2, l",       2, { 0xCB, 0xD5, 0x00, 0x00 } },
  { "set 2, (hl)",    2, { 0xCB, 0xD6, 0x00, 0x00 } },
  { "set 2, (ix+!b)", 3, { 0xDD, 0xCB, 0xD6, 0x00 } },
  { "set 2, (iy+!b)", 3, { 0xFD, 0xCB, 0xD6, 0x00 } },
  { "set 2, a",       2, { 0xCB, 0xD7, 0x00, 0x00 } },
  { "set 3, b",       2, { 0xCB, 0xD8, 0x00, 0x00 } },
  { "set 3, c",       2, { 0xCB, 0xD9, 0x00, 0x00 } },
  { "set 3, d",       2, { 0xCB, 0xDA, 0x00, 0x00 } },
  { "set 3, e",       2, { 0xCB, 0xDB, 0x00, 0x00 } },
  { "set 3, h",       2, { 0xCB, 0xDC, 0x00, 0x00 } },
  { "set 3, l",       2, { 0xCB, 0xDD, 0x00, 0x00 } },
  { "set 3, (hl)",    2, { 0xCB, 0xDE, 0x00, 0x00 } },
  { "set 3, (ix+!b)", 3, { 0xDD, 0xCB, 0xDE, 0x00 } },
  { "set 3, (iy+!b)", 3, { 0xFD, 0xCB, 0xDE, 0x00 } },
  { "set 3, a",       2, { 0xCB, 0xDF, 0x00, 0x00 } },
  { "set 4, b",       2, { 0xCB, 0xE0, 0x00, 0x00 } },
  { "set 4, c",       2, { 0xCB, 0xE1, 0x00, 0x00 } },
  { "set 4, d",       2, { 0xCB, 0xE2, 0x00, 0x00 } },
  { "set 4, e",       2, { 0xCB, 0xE3, 0x00, 0x00 } },
  { "set 4, h",       2, { 0xCB, 0xE4, 0x00, 0x00 } },
  { "set 4, l",       2, { 0xCB, 0xE5, 0x00, 0x00 } },
  { "set 4, (hl)",    2, { 0xCB, 0xE6, 0x00, 0x00 } },
  { "set 4, (ix+!b)", 3, { 0xDD, 0xCB, 0xE6, 0x00 } },
  { "set 4, (iy+!b)", 3, { 0xFD, 0xCB, 0xE6, 0x00 } },
  { "set 4, a",       2, { 0xCB, 0xE7, 0x00, 0x00 } },
  { "set 5, b",       2, { 0xCB, 0xE8, 0x00, 0x00 } },
  { "set 5, c",       2, { 0xCB, 0xE9, 0x00, 0x00 } },
  { "set 5, d",       2, { 0xCB, 0xEA, 0x00, 0x00 } },
  { "set 5, e",       2, { 0xCB, 0xEB, 0x00, 0x00 } },
  { "set 5, h",       2, { 0xCB, 0xEC, 0x00, 0x00 } },
  { "set 5, l",       2, { 0xCB, 0xED, 0x00, 0x00 } },
  { "set 5, (hl)",    2, { 0xCB, 0xEE, 0x00, 0x00 } },
  { "set 5, (ix+!b)", 3, { 0xDD, 0xCB, 0xEE, 0x00 } },
  { "set 5, (iy+!b)", 3, { 0xFD, 0xCB, 0xEE, 0x00 } },
  { "set 5, a",       2, { 0xCB, 0xEF, 0x00, 0x00 } },
  { "set 6, b",       2, { 0xCB, 0xF0, 0x00, 0x00 } },
  { "set 6, c",       2, { 0xCB, 0xF1, 0x00, 0x00 } },
  { "set 6, d",       2, { 0xCB, 0xF2, 0x00, 0x00 } },
  { "set 6, e",       2, { 0xCB, 0xF3, 0x00, 0x00 } },
  { "set 6, h",       2, { 0xCB, 0xF4, 0x00, 0x00 } },
  { "set 6, l",       2, { 0xCB, 0xF5, 0x00, 0x00 } },
  { "set 6, (hl)",    2, { 0xCB, 0xF6, 0x00, 0x00 } },
  { "set 6, (ix+!b)", 3, { 0xDD, 0xCB, 0xF6, 0x00 } },
  { "set 6, (iy+!b)", 3, { 0xFD, 0xCB, 0xF6, 0x00 } },
  { "set 6, a",       2, { 0xCB, 0xF7, 0x00, 0x00 } },
  { "set 7, b",       2, { 0xCB, 0xF8, 0x00, 0x00 } },
  { "set 7, c",       2, { 0xCB, 0xF9, 0x00, 0x00 } },
  { "set 7, d",       2, { 0xCB, 0xFA, 0x00, 0x00 } },
  { "set 7, e",       2, { 0xCB, 0xFB, 0x00, 0x00 } },
  { "set 7, h",       2, { 0xCB, 0xFC, 0x00, 0x00 } },
  { "set 7, l",       2, { 0xCB, 0xFD, 0x00, 0x00 } },
  { "set 7, (hl)",    2, { 0xCB, 0xFE, 0x00, 0x00 } },
  { "set 7, (ix+!b)", 3, { 0xDD, 0xCB, 0xFE, 0x00 } },
  { "set 7, (iy+!b)", 3, { 0xFD, 0xCB, 0xFE, 0x00 } },
  { "set 7, a",       2, { 0xCB, 0xFF, 0x00, 0x00 } },

  { "ret nz",	 1, { 0xC0, 0x00, 0x00, 0x00 } },
  { "ret nc",	 1, { 0xD0, 0x00, 0x00, 0x00 } },
  { "ret po",	 1, { 0xE0, 0x00, 0x00, 0x00 } },
  { "ret p",	  1, { 0xF0, 0x00, 0x00, 0x00 } },
  { "ret z",	  1, { 0xC8, 0x00, 0x00, 0x00 } },
  { "ret c",	  1, { 0xD8, 0x00, 0x00, 0x00 } },
  { "ret pe",	 1, { 0xE8, 0x00, 0x00, 0x00 } },
  { "ret n",	  1, { 0xF8, 0x00, 0x00, 0x00 } },
  { "ret",	    1, { 0xC9, 0x00, 0x00, 0x00 } },
  { "retn",	   2, { 0xED, 0x45, 0x00, 0x00 } },
  { "reti",	   2, { 0xED, 0x4D, 0x00, 0x00 } },

  { "pop bc",	 1, { 0xC1, 0x00, 0x00, 0x00 } },
  { "pop de",	 1, { 0xD1, 0x00, 0x00, 0x00 } },
  { "pop hl",	 1, { 0xE1, 0x00, 0x00, 0x00 } },
  { "pop ix",	 2, { 0xDD, 0xE1, 0x00, 0x00 } },
  { "pop iy",	 2, { 0xFD, 0xE1, 0x00, 0x00 } },
  { "pop af",	 1, { 0xF1, 0x00, 0x00, 0x00 } },
  { "push bc",	1, { 0xC5, 0x00, 0x00, 0x00 } },
  { "push de",	1, { 0xD5, 0x00, 0x00, 0x00 } },
  { "push hl",	1, { 0xE5, 0x00, 0x00, 0x00 } },
  { "push ix",	2, { 0xDD, 0xE5, 0x00, 0x00 } },
  { "push iy",	2, { 0xFD, 0xE5, 0x00, 0x00 } },
  { "push af",	1, { 0xF5, 0x00, 0x00, 0x00 } },

  { "jp (hl)",	1, { 0xE9, 0x00, 0x00, 0x00 } },
  { "jp (ix)",	2, { 0xDD, 0xE9, 0x00, 0x00 } },
  { "jp (iy)",	2, { 0xFD, 0xE9, 0x00, 0x00 } },

  { "rst $00",	1, { 0xC7, 0x00, 0x00, 0x00 } },
  { "rst $08",	1, { 0xCF, 0x00, 0x00, 0x00 } },
  { "rst $10",	1, { 0xD7, 0x00, 0x00, 0x00 } },
  { "rst $18",	1, { 0xDF, 0x00, 0x00, 0x00 } },
  { "rst $20",	1, { 0xE7, 0x00, 0x00, 0x00 } },
  { "rst $28",	1, { 0xEF, 0x00, 0x00, 0x00 } },
  { "rst $30",	1, { 0xF7, 0x00, 0x00, 0x00 } },
  { "rst $38",	1, { 0xFF, 0x00, 0x00, 0x00 } },

  { "im 0",	   2, { 0xED, 0x46, 0x00, 0x00 } },
  { "im 1",	   2, { 0xED, 0x56, 0x00, 0x00 } },
  { "im 2",	   2, { 0xED, 0x5E, 0x00, 0x00 } },

  { "ldi",	    2, { 0xED, 0xA0, 0x00, 0x00 } },
  { "ldd",	    2, { 0xED, 0xA8, 0x00, 0x00 } },
  { "ldir",	   2, { 0xED, 0xB0, 0x00, 0x00 } },
  { "lddr",	   2, { 0xED, 0xB8, 0x00, 0x00 } },

  { "cpi",	    2, { 0xED, 0xA1, 0x00, 0x00 } },
  { "cpd",	    2, { 0xED, 0xA9, 0x00, 0x00 } },
  { "cpir",	   2, { 0xED, 0xB1, 0x00, 0x00 } },
  { "cpdr",	   2, { 0xED, 0xB9, 0x00, 0x00 } },

  { "ini",	    2, { 0xED, 0xA2, 0x00, 0x00 } },
  { "ind",	    2, { 0xED, 0xAA, 0x00, 0x00 } },
  { "inir",	   2, { 0xED, 0xB2, 0x00, 0x00 } },
  { "indr",	   2, { 0xED, 0xBA, 0x00, 0x00 } },

  { "outi",	   2, { 0xED, 0xA3, 0x00, 0x00 } },
  { "outd",	   2, { 0xED, 0xAB, 0x00, 0x00 } },
  { "outir",	  2, { 0xED, 0xB3, 0x00, 0x00 } },
  { "outdr",	  2, { 0xED, 0xBB, 0x00, 0x00 } },

  { "nop",	    1, { 0x00, 0x00, 0x00, 0x00 } },
  { "ex af, 'af",     1, { 0x08, 0x00, 0x00, 0x00 } },
  { "ex de, hl",      1, { 0xEB, 0x00, 0x00, 0x00 } },
  { "ex (sp), hl",    1, { 0xE3, 0x00, 0x00, 0x00 } },
  { "ex (sp), ix",    2, { 0xDD, 0xE3, 0x00, 0x00 } },
  { "ex (sp), iy",    2, { 0xFD, 0xE3, 0x00, 0x00 } },
  { "rlca",	   1, { 0x07, 0x00, 0x00, 0x00 } },
  { "rrca",	   1, { 0x0F, 0x00, 0x00, 0x00 } },
  { "rla",	    1, { 0x17, 0x00, 0x00, 0x00 } },
  { "rra",	    1, { 0x1F, 0x00, 0x00, 0x00 } },
  { "daa",	    1, { 0x27, 0x00, 0x00, 0x00 } },
  { "cpl",	    1, { 0x2F, 0x00, 0x00, 0x00 } },
  { "scf",	    1, { 0x37, 0x00, 0x00, 0x00 } },
  { "ccf",	    1, { 0x3F, 0x00, 0x00, 0x00 } },
  { "halt",	   1, { 0x76, 0x00, 0x00, 0x00 } },
  { "exx",	    1, { 0xD9, 0x00, 0x00, 0x00 } },
  { "di",	     1, { 0xF3, 0x00, 0x00, 0x00 } },
  { "ei",	     1, { 0xFB, 0x00, 0x00, 0x00 } },
  { "neg",	    2, { 0xED, 0x44, 0x00, 0x00 } },

  //alternate opcodes for these commands
  { "*neg",	   2, { 0xED, 0x54, 0x00, 0x00 } },
  { "*neg",	   2, { 0xED, 0x64, 0x00, 0x00 } },
  { "*neg",	   2, { 0xED, 0x74, 0x00, 0x00 } },
  { "*neg",	   2, { 0xED, 0x4C, 0x00, 0x00 } },
  { "*neg",	   2, { 0xED, 0x5C, 0x00, 0x00 } },
  { "*neg",	   2, { 0xED, 0x6C, 0x00, 0x00 } },
  { "*neg",	   2, { 0xED, 0x7C, 0x00, 0x00 } },
  { "*retn",	  2, { 0xED, 0x55, 0x00, 0x00 } },
  { "*retn",	  2, { 0xED, 0x65, 0x00, 0x00 } },
  { "*retn",	  2, { 0xED, 0x75, 0x00, 0x00 } },
  { "*reti",	  2, { 0xED, 0x5D, 0x00, 0x00 } },
  { "*reti",	  2, { 0xED, 0x6D, 0x00, 0x00 } },
  { "*reti",	  2, { 0xED, 0x7D, 0x00, 0x00 } },
  { "*im 0",	  2, { 0xED, 0x66, 0x00, 0x00 } },
  { "*im 1",	  2, { 0xED, 0x76, 0x00, 0x00 } },
  { "*im 0",	  2, { 0xED, 0x4E, 0x00, 0x00 } },
  { "*im 0",	  2, { 0xED, 0x6E, 0x00, 0x00 } },
  { "*im 2",	  2, { 0xED, 0x7E, 0x00, 0x00 } },

  { "ix:",	1, { 0xDD, 0x00, 0x00, 0x00 } },
  { "iy:",	1, { 0xFD, 0x00, 0x00, 0x00 } },
  { "*ed: !1",	1, { 0xED, 0x00, 0x00, 0x00 } },

  { ".db !1",	0, { 0x00, 0x00, 0x00, 0x00 } },
  { ".dw !2",	0, { 0x00, 0x00, 0x00, 0x00 } },
  { ".dr !r",	0, { 0x00, 0x00, 0x00, 0x00 } },
  { NULL,       0, { 0x00, 0x00, 0x00, 0x00 } }
};

#define asm_point	mem_read_byte(_core->mem, loc)
#define asm_pnt(off)	mem_read_byte(_core->mem, loc+(off))

int core_unassemble(core_o core, unsigned int where, FILE *out)
{
	_core_o *_core = (_core_o *)core;
	int operand, i, test, fnd, p;
	int iptr = 0;
	uint16 cnt, loc;
	const char *ts = Z80_instructs[0].id;
	char *ts2, *left, *left2, *right;

	loc = where;

	fprintf(out, "$%04X ", loc);

	if (isprint(asm_point)) {
		cnt = 0;
		while (isprint(asm_pnt(cnt)) && ++cnt);

		if (asm_pnt(cnt) == 0 && cnt >= 3) {
			for (p=0;p<=cnt && p<4;p++)
				fprintf(out, "%02X", asm_pnt(p));

			for (i=p;i<4;i++)
				fprintf(out, "  ");

			fprintf(out, "\t.db \"");
			for (i=0;i<cnt;i++)
				fprintf(out, "%c", asm_pnt(i));
			fprintf(out, "\", 0\n");

			while (p <= cnt) {
				fprintf(out, "$%04X ", loc+p);

				for (i=0;p<=cnt && i<4;i++,p++)
					fprintf(out, "%02X", asm_pnt(p));

				for (;i<4;i++)
					fprintf(out, "  ");

				fprintf(out, "\n");
			}

			return cnt + 1;
		}
	}

	while (ts != NULL) {
		test = 0;

		for (i=0; (i < Z80_instructs[iptr].size) && (i < 2); i++) {
			if (asm_pnt(i) == Z80_instructs[iptr].code[i])
				test++;
		}

		for (; (i < Z80_instructs[iptr].size) && (i < 3); i++) {
			if (asm_pnt(i+1) == Z80_instructs[iptr].code[i])
				test++;
		}

		if (test == Z80_instructs[iptr].size) {
			fnd = _strin(ts, "!b"); //look for nn pre-opcode displacement indicator

			if (fnd != -1) {
				for (i=0; i<Z80_instructs[iptr].size-1; i++,loc++)
					fprintf(out, "%02X", asm_point);

				operand = asm_point;
				loc++;

				fprintf(out, "%02X", (operand & 0xFF));
				fprintf(out, "%02X", asm_point);

				for (i++;i<4;i++)
					fprintf(out, "  ");

				left = _left(ts, fnd);
				right = _right(ts, fnd+2);

				fprintf(out, "\t%s$%02X%s\n", left, operand, right);

				free(right);
				free(left);

				return Z80_instructs[iptr].size+1;
      			}

			fnd = _strin(ts, "!d"); //look for nn displacement indicator
			if (fnd != -1) {
				ts2 = _right(ts, fnd+2);

				p = _strin(ts2, "!1");
				if (p != -1) {
					for (i=0; i<Z80_instructs[iptr].size; i++,loc++)
						fprintf(out, "%02X", asm_point);

					operand = asm_point;
					loc++;

					fprintf(out, "%02X", (operand & 0xFF));
	  				fprintf(out, "%02X", (asm_point & 0xFF));

					for (i+=2; i<4; i++)
						fprintf(out, "  ");

					left = _left(ts, fnd);
					left2 = _left(ts2, p);
					right = _right(ts2, p+2);

					fprintf(out, "\t%s$%02X%s$%02X%s\n",
						left, operand, left2, (asm_point & 0xFF), right);

					free(right);
					free(left2);
					free(left);
					free(ts2);

					return Z80_instructs[iptr].size + 2;
				}
				free(ts2);

				for (i=0;i<Z80_instructs[iptr].size;i++,loc++)
					fprintf(out, "%02X", asm_point);

				operand = asm_point;

				fprintf(out, "%02X", (operand & 0xFF));

				for (i++;i<4;i++)
					fprintf(out, "  ");

				left = _left(ts, fnd);
				right = _right(ts, fnd + 2);

				fprintf(out, "\t%s$%02X%s\n", left, operand, right);

				free(left);
				free(right);

				return Z80_instructs[iptr].size+1;
			}

			fnd = _strin(ts, "!r"); //look for nn relative immediate indicator
			if (fnd != -1) {
				for (i=0; i<Z80_instructs[iptr].size; i++,loc++)
					fprintf(out, "%02X", asm_point);

				operand = loc + 1 + (signed char)(asm_point);

				fprintf(out, "%02X", asm_point);

				for (i++; i<4; i++)
					fprintf(out, "  ");

				left = _left(ts, fnd);
				right = _right(ts, fnd+2);

				fprintf(out, "\t%s$%04X%s\n", left, operand, right);

				free(right);
				free(left);

				return Z80_instructs[iptr].size+1;
			}

			fnd = _strin(ts, "!2"); //look for nnnn immediate indicator
			if (fnd != -1) {
				for (i=0; i<Z80_instructs[iptr].size; i++,loc++)
					fprintf(out, "%02X", asm_point);

				operand = asm_point; loc++;
				operand += (asm_point << 8);
				fprintf(out, "%02X%02X", (operand & 0xFF), (operand >> 8));

				for (i+=2; i<4; i++)
					fprintf(out, "  ");

				left = _left(ts, fnd);
				right = _right(ts, fnd+2);

				fprintf(out, "\t%s$%04X%s\n", left, operand, right);

				free(right);
				free(left);

				return Z80_instructs[iptr].size+2;
			}

			fnd = _strin(ts, "!1"); //look for nn immediate indicator
			if (fnd != -1) {
				for (i=0; i<Z80_instructs[iptr].size; i++,loc++)
					fprintf(out, "%02X", asm_point);

				operand = asm_point;
				fprintf(out, "%02X", (operand & 0xFF));

				for (i++; i<4; i++)
					fprintf(out, "  ");

				left = _left(ts, fnd);
				right = _right(ts, fnd+2);

				fprintf(out, "\t%s$%02X%s\n", left, operand, right);

				free(right);
				free(left);

				return Z80_instructs[iptr].size+1;
			}

			for (i=0; i<Z80_instructs[iptr].size; i++,loc++)
				fprintf(out, "%02X", asm_point);

			for (; i<4; i++)
				fprintf(out, "  ");

			fprintf(out, "\t%s\n", ts);
			return Z80_instructs[iptr].size;
		}

		iptr++;
		ts = Z80_instructs[iptr].id;
	}

	fprintf(out, "%02X\t.db $%02X\n", asm_point, asm_point);
	return 1;
}


