#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>

#define VPCALC_SECTION	"LIB"
#define VPCALC_SUBSECT	"CPU"

#include "config.h"
#include "../include/types.h"
#include "../include/debug.h"

#include "../include/timing.h"

#include "../include/mem.h"
#include "../include/gfx.h"
#include "../include/dev.h"
#include "../include/cpu.h"
#include "../include/core.h"

typedef struct _cpu_o {
	void *library;

	core_o core;

	void (*del)(core_o core);

	int (*execute)(core_o core);
	void (*terminate)(core_o core);

	int  (*cycle)(core_o core);
	void (*interrupt)(core_o core, unsigned int data);

	void (*call)(core_o core, unsigned int addr);
	void (*ret)(core_o core);

	void (*report)(core_o core, FILE *out);
	void (*alter)(core_o core, const char *regname, unsigned int value);
} _cpu_o;

cpu_o cpu_new(const char *libname, mem_o mem, dev_o dev, timing_o *rate)
{
	_cpu_o *new;
	core_o (*core_new)(mem_o, dev_o, timing_o *rate);
	const char *error;
  
	new = (_cpu_o *) malloc(sizeof(_cpu_o));
	if (new == NULL) {
		_err_report("Unable to allocate cpu structure.");

		return NULL;
	}

	new->library = dlopen(libname, RTLD_NOW);
	if (!new->library) {
		error = dlerror();
		_err_report(error);
		free(new);
		return NULL;
	}

	core_new = dlsym(new->library, "core_new");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->del = dlsym(new->library, "core_del");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->execute = dlsym(new->library, "core_execute");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->terminate = dlsym(new->library, "core_terminate");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->cycle = dlsym(new->library, "core_cycle");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->interrupt = dlsym(new->library, "core_interrupt");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->call = dlsym(new->library, "core_call");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->ret = dlsym(new->library, "core_ret");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->report = dlsym(new->library, "core_report");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->alter = dlsym(new->library, "core_alter");
	error = dlerror();
	if (error != NULL) {
		_err_report(error);

		dlclose(new->library);
		free(new);
		return NULL;
	}

	new->core = core_new(mem, dev, rate);
	if (new->core == NULL) {
		_err_report("failed to make new core.");

		dlclose(new->library);
		free(new);
		return NULL;
	}

	return (cpu_o)new;
}

void cpu_del(cpu_o cpu)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	_cpu->del(_cpu->core);

	dlclose(_cpu->library);
	free(_cpu);
}

int cpu_execute(cpu_o cpu)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	return _cpu->execute(_cpu->core);
}

void cpu_terminate(cpu_o cpu)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	_cpu->terminate(_cpu->core);
}

int cpu_cycle(cpu_o cpu)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	return _cpu->cycle(_cpu->core);
}

void cpu_interrupt(cpu_o cpu, unsigned int data)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	_cpu->interrupt(_cpu->core, data);
}

void cpu_call(cpu_o cpu, unsigned int addr)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	_cpu->call(_cpu->core, addr);
}

void cpu_ret(cpu_o cpu)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	_cpu->ret(_cpu->core);
}

void cpu_report(cpu_o cpu, FILE *out)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	_cpu->report(_cpu->core, out);
}

void cpu_alter(cpu_o cpu, const char *regname, unsigned int value)
{
	_cpu_o *_cpu = (_cpu_o *)cpu;

	_cpu->alter(_cpu->core, regname, value);
}
